/* Treat Trough of Terror
  2021 - Gord Payne

  Build a Treat-displensing slide for Halloween Trick-or-treaters!
  Includes Addressable LED lights and MP3 Player for scary sounds!
*/

#include "SoftwareSerial.h" // to communicate with MP3 player
#include "DFRobotDFPlayerMini.h" // for MP3 player

SoftwareSerial mp3Player(7, 8);// RX7, TX8 MP3 Player
DFRobotDFPlayerMini myDFPlayer;


#include <FastLED.h> // library for Addressable LED strip

#define LED_PIN     11  // match your Arduio setup
#define NUM_LEDS    240 // match your LED strip length/density
#define BRIGHTNESS  32
#define LED_TYPE    WS2811
#define COLOR_ORDER GRB
CRGB leds[NUM_LEDS];

#define UPDATES_PER_SECOND 60
int led = 13;

#define echoPin 2 // attach pin D2 Arduino to pin Echo of HC-SR04
#define trigPin 3 //attach pin D3 Arduino to pin Trig of HC-SR04

// defines variables
long duration; // variable for the duration of sound wave travel on SR-04
int distance; // variable for the distance measurement



CRGBPalette16 currentPalette;
TBlendType    currentBlending;

extern CRGBPalette16 myRedWhiteBluePalette;
extern const TProgmemPalette16 myRedWhiteBluePalette_p PROGMEM;
char theVal;

void setup() {
  Serial.begin(115200);
  pinMode(trigPin, OUTPUT); // Sets the trigPin as an OUTPUT
  pinMode(echoPin, INPUT); // Sets the echoPin as an INPUT

  delay( 3000 ); // power-up safety delay
  FastLED.addLeds<LED_TYPE, LED_PIN, COLOR_ORDER>(leds, NUM_LEDS).setCorrection( TypicalLEDStrip );
  FastLED.setBrightness(  BRIGHTNESS );

  currentPalette = RainbowColors_p;
  currentBlending = LINEARBLEND;

  //MP3 initialization
  mp3Player.begin(9600);

  while (!myDFPlayer.begin(mp3Player)) {  //Use softwareSerial to communicate with mp3.
    Serial.println(F("Unable to begin:"));
    Serial.println(F("1.Please recheck the connection!"));
    Serial.println(F("2.Please insert the SD card!"));
    delay(100);

  }
  myDFPlayer.volume(30);
  Serial.println(F("DFPlayer Mini online."));
  Serial.println("setup done");
}


void loop()
{
  // Clears the trigPin condition
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  // Sets the trigPin HIGH (ACTIVE) for 10 microseconds
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  // Reads the echoPin, returns the sound wave travel time in microseconds
  duration = pulseIn(echoPin, HIGH);
  // Calculating the distance
  distance = duration * 0.034 / 2; // Speed of sound wave divided by 2 (go and back)
  if (distance < 8.0) { // sequence LEDs and sound if your hand is within 8cm of the sensor
    // generate random value 1 to 8 inclusive
    int ndx = int(random(1, 9));

    myDFPlayer.play(ndx);
    FastLED.setBrightness(  BRIGHTNESS );
    for (float i = 0; i < NUM_LEDS / 2; i++) {
      ChangePalettePeriodically();
      static uint8_t startIndex = 0;
      startIndex = startIndex + 4; //motion speed
      FillLEDsFromPaletteColors( startIndex);
      FastLED.show();
      FastLED.delay(1000 / UPDATES_PER_SECOND);
    }
    FastLED.setBrightness(  0 );
    FastLED.show();
  }
}

void FillLEDsFromPaletteColors( uint8_t colorIndex)
{
  uint8_t brightness = 255;

  for ( int i = NUM_LEDS / 2; i > 0; i--) {
    leds[i] = ColorFromPalette( currentPalette, colorIndex, brightness, currentBlending);
    leds[NUM_LEDS - i - 1] = ColorFromPalette( currentPalette, colorIndex, brightness, currentBlending);

    colorIndex += 3;
  }
}

void ChangePalettePeriodically()
{
  SetupOrangeAndYellowPalette();
  currentBlending = LINEARBLEND;
}

// This function sets up a palette of orange and yellow stripes.
void SetupOrangeAndYellowPalette()
{
  CRGB orange = CHSV( HUE_ORANGE, 255, 255);
  CRGB yellow  = CHSV( HUE_RED, 255, 255);
  CRGB black  = CRGB::Black;

  currentPalette = CRGBPalette16(// swap in some yellows if desired.
                     orange,  orange,  black,  black,
                     orange, orange, black,  black,
                     orange,  orange,  black,  black,
                     orange, orange, black,  black );
}

void playTheFile(int theChoice) {
  myDFPlayer.volume(30);
  myDFPlayer.play(theChoice);
  delay(500);
}
